@@ -30,13 +30,13 @@
 #define OPAL_RELEASE_VERSION 9
 #define OPAL_GREEK_VERSION "a1"
 #define OPAL_WANT_REPO_REV @OPAL_WANT_REPO_REV@
-#define OPAL_REPO_REV "v4.1.5-222-g92f9fca4eb"
+#define OPAL_REPO_REV "v4.1.5-232-gad48c462ff"
 #ifdef OPAL_VERSION
 /* If we included version.h, we want the real version, not the
    stripped (no-r number) verstion */
 #undef OPAL_VERSION
 #endif
 #define OPAL_VERSION "4.1.9a1"
-#define OPAL_CONFIGURE_CLI " \'CC=gcc\' \'CXX=g++\' \'FC=nvfortran\' \'LDFLAGS=-Wl,-rpath-link=/proj/nv/libraries/Linux_x86_64/25.9/hpcx-13/284675-rel-1/comm_libs/13.0/hpcx/hpcx-2.24/ucx/lib -Wl,-rpath-link=/proj/nv/libraries/Linux_x86_64/25.9/hpcx-13/284675-rel-1/comm_libs/13.0/hpcx/hpcx-2.24/hcoll/lib\' \'--with-platform=../contrib/platform/nvhpc/optimized\' \'--enable-mpi1-compatibility\' \'--with-libevent=internal\' \'--without-xpmem\' \'--without-cray-xpmem\' \'--with-slurm\' \'--with-cuda=/proj/cuda/13.0/Linux_x86_64\' \'--with-hcoll=/proj/nv/libraries/Linux_x86_64/25.9/hpcx-13/284675-rel-1/comm_libs/13.0/hpcx/hpcx-2.24/hcoll\' \'--with-ucc=/proj/nv/libraries/Linux_x86_64/25.9/hpcx-13/284675-rel-1/comm_libs/13.0/hpcx/hpcx-2.24/ucc\' \'--with-ucx=/proj/nv/libraries/Linux_x86_64/25.9/hpcx-13/284675-rel-1/comm_libs/13.0/hpcx/hpcx-2.24/ucx\' \'--prefix=/proj/nv/libraries/Linux_x86_64/25.9/hpcx-13/284675-rel-1/comm_libs/13.0/hpcx/hpcx-2.24/ompi\'"
+#define OPAL_CONFIGURE_CLI " \'CC=gcc\' \'CXX=g++\' \'FC=nvfortran\' \'LDFLAGS=-Wl,-rpath-link=/proj/nv/libraries/Linux_x86_64/25.11/hpcx-13/289927-rel-1/comm_libs/13.0/hpcx/hpcx-2.25.1/ucx/lib -Wl,-rpath-link=/proj/nv/libraries/Linux_x86_64/25.11/hpcx-13/289927-rel-1/comm_libs/13.0/hpcx/hpcx-2.25.1/hcoll/lib\' \'--with-platform=../contrib/platform/nvhpc/optimized\' \'--enable-mpi1-compatibility\' \'--with-libevent=internal\' \'--without-xpmem\' \'--without-cray-xpmem\' \'--with-slurm\' \'--with-cuda=/proj/cuda/13.0/Linux_x86_64\' \'--with-hcoll=/proj/nv/libraries/Linux_x86_64/25.11/hpcx-13/289927-rel-1/comm_libs/13.0/hpcx/hpcx-2.25.1/hcoll\' \'--with-ucc=/proj/nv/libraries/Linux_x86_64/25.11/hpcx-13/289927-rel-1/comm_libs/13.0/hpcx/hpcx-2.25.1/ucc\' \'--with-ucx=/proj/nv/libraries/Linux_x86_64/25.11/hpcx-13/289927-rel-1/comm_libs/13.0/hpcx/hpcx-2.25.1/ucx\' \'--prefix=/proj/nv/libraries/Linux_x86_64/25.11/hpcx-13/289927-rel-1/comm_libs/13.0/hpcx/hpcx-2.25.1/ompi\'"
 
 #endif
