@@ -317,7 +317,8 @@ class AggNodeFabricInfo : public AggNode
     void FillTreeEdgesVec(uint32_t port_num, VecTreeEdgePtr& tree_edges);
 
     void SetPriorityEpoch(uint64_t epoch);
-    uint64_t GetPriorityEpoch() { return m_priority_epoch_; }
+    bool IsLowPriority() const { return 0 != m_priority_epoch_; }
+    uint64_t GetPriorityEpoch() const { return m_priority_epoch_; }
 
     bool IsLockAvailable() const;
     bool IsExclusiveLockAvailable() const;
@@ -353,7 +354,7 @@ class AggNodeFabricInfo : public AggNode
     uint8_t GetDfpTreeHeight(uint8_t plane);
     uint16_t GetGroupNaiborsNumber(uint8_t plane) const;
 
-    void SetCleanOrUnconfigFailed(uint64_t epoch);
+    void SetCleanOrUnconfigFailed(uint64_t epoch);   // put the node in Pending Cleanup state
 
    private:
     QpStateEnum QPCConfigStateToQpStateEnum(int state)
