@@ -425,6 +425,36 @@ class SmsRecord
     bool SetState(const char* field_str) { return CsvParser::Parse(field_str, state, 10); }
 };
 
+class PortPkeyRecord
+{
+   public:
+    u_int64_t port_guid;
+    std::set<u_int16_t> pkeys;
+
+    PortPkeyRecord() : port_guid(0), pkeys() {}
+
+    static int Init(vector<ParseFieldInfo<class PortPkeyRecord>>& parse_section_info);
+
+    bool SetPortGuid(const char* field_str) { return CsvParser::Parse(field_str, port_guid, 16); }
+
+    bool SetPkeys(const char* field_str);
+};
+
+class VportPkeyRecord
+{
+   public:
+    u_int64_t vport_guid;
+    std::set<u_int16_t> pkeys;
+
+    VportPkeyRecord() : vport_guid(0), pkeys() {}
+
+    static int Init(vector<ParseFieldInfo<class VportPkeyRecord>>& parse_section_info);
+
+    bool SetVportGuid(const char* field_str) { return CsvParser::Parse(field_str, vport_guid, 16); }
+
+    bool SetPkeys(const char* field_str);
+};
+
 typedef std::map<AnToAnKey, AnToAnInfo> MapAnToAnInfo;
 typedef MapAnToAnInfo::iterator MapAnToAnInfoIter;
 typedef std::pair<MapAnToAnInfoIter, bool> MapAnToAnInfoInsertRes;
@@ -496,6 +526,7 @@ class Port
     u_int16_t m_aport_;
     u_int8_t m_plane_number_;
     u_int8_t m_number_of_planes_;
+    Pkeys_t m_pkeys_;
 
     Epoch m_port_epoch_;   // last update number
 
@@ -594,6 +625,9 @@ class Port
     void IncMadErrorCount() { m_error_count++; }
     void ResetMadErrorCount() { m_error_count = 0; }
 
+    inline void SetPkeys(const Pkeys_t& pkeys) { m_pkeys_ = pkeys; }
+    inline const Pkeys_t& GetPkeys() const { return m_pkeys_; }
+
    private:
     // disconnect the port and remote port.
     void Disconnect();
@@ -673,6 +707,7 @@ class Vnode
     uint64_t m_guid_;          // The virtual node GUID
     string m_description_;     // Description of the vnode
     uint64_t m_vnode_epoch_;   // last update number
+    Pkeys_t m_pkeys_;
 
    public:
     // constructor
@@ -685,6 +720,8 @@ class Vnode
     inline const string& GetDescription() const { return m_description_; }
     inline void UpdateDescription(const std::string& new_description) { m_description_ = new_description; }
     inline uint64_t GetGuid() const { return m_guid_; }
+    inline void SetPkeys(const Pkeys_t& pkeys) { m_pkeys_ = pkeys; }
+    inline const Pkeys_t& GetPkeys() const { return m_pkeys_; }
 };
 
 enum UpDownMinhopDirection
