@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2012-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2012-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -58,6 +59,13 @@
 
 #define QPC_QP_TRANSPORT_TYPE_UD 3   // UD - Unreliable Datagram
 
+#define MULTICAST_PORT_GROUP_SIZE       256
+#define MULTICAST_MAX_PORT_GROUP_IDX    1   // support up to 2*256 ports
+#define MULTICAST_BLOCK_SIZE            4
+#define MULTICAST_FIRST_MLID            0xC000
+#define MULTICAST_PORT_GROUPS_NUMBER    (MAX_PORTS_NUM / MULTICAST_PORT_GROUP_SIZE)
+#define GET_MULTICAST_BLOCK_INDEX(mlid) ((mlid - MULTICAST_FIRST_MLID) / MULTICAST_BLOCK_SIZE)
+
 template <typename _Type, void (_Type::*_Method)(const clbck_data_t&, int, void*)>
 void ibis_forwardClbck(const clbck_data_t& clbck_data, int rec_status, void* p_attribute_data)
 {
@@ -85,6 +93,7 @@ struct sharp_quota;
 struct PortInfo;
 struct FabricProviderCallbackContext;
 struct qp_record;
+class AMBitArray;
 
 typedef std::list<uint32_t> QpList;
 typedef std::pair<uint8_t, uint32_t> ChildQp;
@@ -218,12 +227,26 @@ class AnAdapter
                                       MAD_AggregationManagement* p_am_mad,
                                       Notice* p_notice);
 
-    virtual void HandleTrap(TrapHandlingCallback traps_handling_callback,
-                            void* p_traps_callback_context,
-                            SharpTrapNumberEnum trap_number,
-                            ib_address_t* p_ib_address,
-                            MAD_AggregationManagement* p_am_mad,
-                            Notice* p_notice);
+    virtual void HandleTrapSharpQPAllocationTime(TrapHandlingCallback traps_handling_callback,
+                                                 void* p_traps_callback_context,
+                                                 u_int8_t* p_notice_data_details,
+                                                 ib_address_t* p_ib_address,
+                                                 MAD_AggregationManagement* p_am_mad,
+                                                 Notice* p_notice) = 0;
+
+    virtual void HandleTrapSharpAMKeyViolation(TrapHandlingCallback traps_handling_callback,
+                                               void* p_traps_callback_context,
+                                               ib_address_t* p_ib_address,
+                                               MAD_AggregationManagement* p_am_mad,
+                                               Notice* p_notice,
+                                               AM_TrapAMKeyViolation& trap_am_key_violation);
+
+    virtual void HandleUnsupportedTrap(TrapHandlingCallback traps_handling_callback,
+                                       void* p_traps_callback_context,
+                                       SharpTrapNumberEnum trap_number,
+                                       ib_address_t* p_ib_address,
+                                       MAD_AggregationManagement* p_am_mad,
+                                       Notice* p_notice);
 };
 
 class AnAdapterV1 : public AnAdapter
@@ -254,6 +277,13 @@ class AnAdapterV1 : public AnAdapter
                                                ib_address_t* p_ib_address,
                                                MAD_AggregationManagement* p_am_mad,
                                                Notice* p_notice);
+
+    virtual void HandleTrapSharpQPAllocationTime(TrapHandlingCallback traps_handling_callback,
+                                                 void* p_traps_callback_context,
+                                                 u_int8_t* p_notice_data_details,
+                                                 ib_address_t* p_ib_address,
+                                                 MAD_AggregationManagement* p_am_mad,
+                                                 Notice* p_notice);
 };
 
 class AnAdapterV2 : public AnAdapter
@@ -284,6 +314,13 @@ class AnAdapterV2 : public AnAdapter
                                                ib_address_t* p_ib_address,
                                                MAD_AggregationManagement* p_am_mad,
                                                Notice* p_notice);
+
+    virtual void HandleTrapSharpQPAllocationTime(TrapHandlingCallback traps_handling_callback,
+                                                 void* p_traps_callback_context,
+                                                 u_int8_t* p_notice_data_details,
+                                                 ib_address_t* p_ib_address,
+                                                 MAD_AggregationManagement* p_am_mad,
+                                                 Notice* p_notice);
 };
 
 struct SrDeleter
@@ -305,11 +342,12 @@ class FabricProvider
           m_stop_listener_flag_(true),
           m_sr_lease_time_(sr_lease_time),
           m_sr_context_(nullptr),
-          m_port_invalid_time{.tv_sec = 0, .tv_usec = 0},
+          m_port_invalid_time{.tv_sec = 0, .tv_nsec = 0},
           m_event_listener_started(false),
           m_traps_handling_callback_(NULL),
           m_traps_callback_context_(NULL),
-          m_operation_mode_(op_mode){};
+          m_operation_mode_(op_mode),
+          m_max_mlid_(MULTICAST_FIRST_MLID){};
 
     ~FabricProvider();
 
@@ -346,13 +384,15 @@ class FabricProvider
 
     void SendRepressMsg(ib_address_t* p_ib_address, MAD_AggregationManagement* p_am_mad, Notice* p_noice);
 
-    int EventListenerStart(int (*get_address_cb)(uint8_t*, struct sr_addr_info*));
+    int EventListenerStart(int (*get_address_cb)(uint8_t*, struct sr_addr_info*), uint64_t sa_key);
     int EventListenerStop();
 
     static void RegisterAddress(const void* delegate, void* context);
 
     void WaitForPendingTransactions();
 
+    void SetMaxMlid(lid_t mlid) { m_max_mlid_ = (mlid > MULTICAST_FIRST_MLID ? mlid : MULTICAST_FIRST_MLID); }
+
     int SetAMKey(const AggPortInfo& agg_port_info,
                  void (*f)(FabricProviderCallbackContext* callback_context, int rec_status, void* p_attribute_data),
                  FabricProviderCallbackContext* callback_context,
@@ -374,7 +414,7 @@ class FabricProvider
 
     // Bind TreeIds to JobId
     int BindTreeIdsToJob(const AggPortInfo& agg_port_info,
-                         vector<uint16_t> tree_ids_to_bind,
+                         const vector<uint16_t>& tree_ids_to_bind,
                          int offset,
                          sharp_job_id_t job_id,
                          void (*f)(FabricProviderCallbackContext* callback_context, int rec_status, void* p_attribute_data),
@@ -383,7 +423,7 @@ class FabricProvider
     // Get binded TreeIds to JobId
     int SendTreeToJobBindingGet(const AggPortInfo& agg_port_info,
                                 sharp_job_id_t job_id,
-                                VecTreeIds tree_ids_to_check,
+                                VecTreeIds& tree_ids_to_check,
                                 void (*f)(FabricProviderCallbackContext* callback_context, int rec_status, void* p_attribute_data),
                                 FabricProviderCallbackContext* callback_context);
 
@@ -463,6 +503,7 @@ class FabricProvider
                     bool is_sat,
                     uint16_t pkey,
                     SharpMtu mtu,
+                    uint8_t plane,
                     void (*f)(FabricProviderCallbackContext* callback_context, int rec_status, void* p_attribute_data),
                     FabricProviderCallbackContext* callback_context);
 
@@ -585,12 +626,33 @@ class FabricProvider
 
     uint64_t GetPortGuid() { return m_ibis_port_properties_.port_guid; }
 
-    bool IsPortUp() { return (m_ibis_port_properties_.state > IBIS_IB_PORT_STATE_DOWN); }
+    bool IsPortUp() { return (m_ibis_port_properties_.state == IBV_PORT_ACTIVE); }
 
     void UpdatePortProperties();
     int StartServiceRecordRdma(int (*get_address_cb)(uint8_t*, struct sr_addr_info*), port_key_t port_key);
     void StopServiceRecordRdma();
 
+    int UpdateMulticastTable(MulticastForwardingData& multicast_data, lid_t mlid, AMBitArray& down_ports, phys_port_t up_port);
+
+    int ClearMulticastTable(MulticastForwardingData& multicast_data, lid_t mlid);
+
+    void CleanAllMulticastTables(MulticastForwardingData& multicast_data);
+
+    int ConfigureMulticastTable(const AggPortInfo& agg_port_info,
+                                MulticastForwardingData& multicast_data,
+                                fabric_provider_handle_data_func_t callback_dlg,
+                                FabricProviderCallbackContext* callback_context);
+
+    int ReadRequiredMulticastTables(const AggPortInfo& agg_port_info,
+                                    MulticastForwardingData& multicast_data,
+                                    ListLids& mlids,
+                                    fabric_provider_handle_data_func_t callback_dlg,
+                                    FabricProviderCallbackContext* callback_context);
+
+    bool IsMlidConfiguredOnNodeData(MulticastForwardingData& multicast_data, lid_t mlid);
+
+    void GetMulticastTablePorts(MulticastForwardingData& multicast_data, lid_t mlid, SetPhysPorts& down_ports, SetPhysPorts& up_ports);
+
    private:
     Ibis m_ibis_mads_;
     Ibis m_ibis_traps_;
@@ -602,7 +664,7 @@ class FabricProvider
 
     int m_sr_lease_time_;
     std::unique_ptr<struct sr_ctx, SrDeleter> m_sr_context_;
-    struct timeval m_port_invalid_time;
+    struct timespec m_port_invalid_time;
 
     FdEventListener m_fd_event_listener_;
     bool m_event_listener_started;
@@ -613,6 +675,8 @@ class FabricProvider
 
     FabricProviderOpMode m_operation_mode_;
 
+    lid_t m_max_mlid_;   //  Maximal mlid handled by multicast tables
+
     // Disable compiler generated copy-assignment operators
     FabricProvider& operator=(const FabricProvider&);
     FabricProvider(const FabricProvider&);
@@ -770,6 +834,28 @@ class FabricProvider
 
     MemoryPool<FpCallbackData> fp_callback_data_pool;
 
+    struct IbMulticastTableEntry : public MulticastTableEntryI
+    {
+        u_int8_t m_disable_lbf_mask = 0;
+        u_int8_t m_set_disable_lbf_mask = 0;
+        u_int8_t m_inconsistent_lbf_cfg_mask = 0;
+        struct UINT256 m_port_group[MULTICAST_BLOCK_SIZE] = {};
+        uint8_t m_used_mlids = 0;
+
+        virtual ~IbMulticastTableEntry() {}
+        int init()
+        {
+            m_disable_lbf_mask = 0;
+            m_set_disable_lbf_mask = 0;
+            m_inconsistent_lbf_cfg_mask = 0;
+            memset(m_port_group, 0, sizeof(m_port_group));
+            m_used_mlids = 0;
+            return 0;
+        }
+    };
+
+    MemoryPool<IbMulticastTableEntry> mc_table_entry_pool_;
+
     // Aggregation nodes discovery process functions
 
     struct sr_ctx* CreateSrContext(struct sr_config& conf);
@@ -911,6 +997,34 @@ class FabricProvider
     int QueryTreeConfigNextRecord(FpCallbackData* fcd, uint32_t record_locator);
     int QueryQPDatabaseNextRecord(FpCallbackData* fcd, uint32_t record_locator);
 
+    IbMulticastTableEntry* GetOrAllocateMulticastTableEntry(MulticastForwardingData* p_multicast_data,
+                                                            uint16_t block_idx,
+                                                            uint8_t port_group,
+                                                            uint8_t plft,
+                                                            bool allocate_expected);
+
+    int UpdateMulticastTableBlock(MulticastForwardingData& multicast_data,
+                                  lid_t mlid,
+                                  uint16_t block_idx,
+                                  uint8_t port_group,
+                                  uint8_t plft,
+                                  bool disable_lbf,
+                                  struct UINT256& port_group_data);
+
+    int SendSetMulticastTableRequest(FpCallbackData* fcd,
+                                     void (*f)(const clbck_data_t& clbck_data, int rec_status, void* p_attribute_data),
+                                     MulticastForwardingData* p_multicast_data);
+
+    void ConfigureMulticastTableCallback(const clbck_data_t& clbck_data, int rec_status, void* p_attribute_data);
+
+    int SendGetMulticastTablesRequest(FpCallbackData* fcd,
+                                      void (*f)(const clbck_data_t& clbck_data, int rec_status, void* p_attribute_data),
+                                      MulticastForwardingData* p_multicast_data);
+
+    void GetMulticastTablesCallback(const clbck_data_t& clbck_data, int rec_status, void* p_attribute_data);
+
+    void GetMulticastTablePortsByPlft(MulticastForwardingData& multicast_data, lid_t mlid, SetPhysPorts& ports, uint8_t plft);
+
     friend class AnAdapterV1;
     friend class AnAdapterV2;
 };
