@@ -148,6 +148,9 @@ class SharpJob
 
     SharpTimestamp m_start_time_{SharpTimestamp::clock::now()};
 
+    // Rounding mode used by this job - needed for conflict detection
+    sharp_round_mode m_round_mode_;
+
     int FindCoalescingRails(const sharp_begin_job& begin_job_data,
                             vector<uint8_t>& coalescing_rails,
                             VecVecPortData& vec_rail_to_vec_port_data,
@@ -276,6 +279,8 @@ class SharpJob
     void SetJobInfoSendTimeStamp(bool job_info_send);
     void SetJobInfoReplyReceived(bool job_info_reply_received) { m_job_info_reply_received = job_info_reply_received; }
 
+    sharp_round_mode GetRoundMode() const { return m_round_mode_; }
+
     sharp_am_status FabricTreesConfig();
     sharp_am_status FabricQuotaConfig();
     void FabricJobResourceCleanup();
