@@ -1,6 +1,6 @@
 /**
  * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2015-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2015-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * See file LICENSE for terms.
  */
 
@@ -67,6 +67,7 @@ const char* comm_type_names[] = {
 static struct option long_options[] = {{"help", no_argument, NULL, 'h'},
                                        {"perf_with_barrier", required_argument, NULL, 'B'},
                                        {"collectives", required_argument, NULL, 'c'},
+                                       {"rs_ag_mode", required_argument, NULL, 'R'},
                                        {"iov_count", required_argument, NULL, 'C'},
                                        {"ib-dev", required_argument, NULL, 'd'},
                                        {"data_layout", required_argument, NULL, 'D'},
@@ -96,13 +97,14 @@ static const char usage_string[] =
     "-i, --iters             Number of iterations to run perf benchmark (Default: 100)\n"
     "-x, --skips             Number of warmup iterations to run perf benchmark (Default: 10)\n"
     "-M, --mem_type          Memory type(host,cuda,null) used in the communication buffers format: <src memtype>:<recv memtype> \n"
-    "                        (Dafault: host cpu memory on non-GPU setup. cuda memory for GPU setup\n"
+    "                        (Dafault: host cpu memory on non-GPU setup. cuda memory for GPU setup)\n"
     "--use_cuda_dmabuf       Use CUDA DMA-BUF for GPUDirect RDMA testing with CUDA memory\n"
     "--use_data_direct       Use Data-Direct CUDA DMA-BUF for GPUDirect RDMA testing\n"
 #ifdef MPI_SUPPORT
     "-B, --perf_with_barrier Sync Allreduce with Barrier in Allreduce collective, Default:1 \n"
-    "-c, --collectives       Comma separated list of collectives:[allreduce|iallreduce|reduce|ireduce|barrier|ibarrier|bcast|reduce_scatter|allgather|all|none] to run.\n"
+    "-c, --collectives       Comma separated list of collectives:[allreduce|iallreduce|reduce|ireduce|barrier|ibarrier|bcast|reduce_scatter|allgather|rs_ag|all|none] to run.\n"
     "                           Default: Allreduce\n"
+    "-R, --rs_ag_mode        reduce-scatter and allgather chaining mode: seq|ovl (sequential or overlapped). Default: seq\n"
     "-H, --host_alloc_type   Host memory allocation method ( hugetlb, hugethp(MADVISE Transparent Hugepage support), malloc)\n"
     "-m, --mode              Test modes: <basic|complex|perf|all> . Default: perf\n"
     "-N, --nbc_count         Number of non-blocking operation posted before waiting for completion (max: 512)\n"
@@ -434,6 +436,8 @@ static void sharp_run_perf(coll_sharp_module_t* sharp_comm)
         coll_test_sharp_reduce_scatter_perf(sharp_comm);
     if (sharp_conf.run_colls & TEST_ALLGATHER)
         coll_test_sharp_allgather_perf(sharp_comm);
+    if (sharp_conf.run_colls & TEST_RS_AG)
+        coll_test_sharp_rs_ag_perf(sharp_comm, sharp_conf.rs_ag_overlapped);
 }
 
 void coll_module_op(coll_sharp_module_t* sharp_comm)
@@ -610,14 +614,14 @@ int sharp_test_read_memunits(const void* str, size_t* dest, ssize_t low, ssize_t
     value *= bytes;
 
     if (value < low || value > high) {
-        fprintf(stderr, "%s %lu - %lu\n", "Invalid range of mememory size. expected range ", low, high);
+        fprintf(stderr, "%s %lu - %lu\n", "Invalid range of memory size. expected range ", low, high);
         return 1;
     }
 
     *dest = value;
     return 0;
 err_memunits:
-    fprintf(stderr, "%s: %s\n", (char*)str, "Invalid memunits. syntax:<number>[b|kb|mb|gb] or \"inf\"");
+    fprintf(stderr, "%s: %s\n", (char*)str, "Invalid memory units. syntax:<number>[b|kb|mb|gb] or \"inf\"");
     return 1;
 }
 
@@ -633,7 +637,7 @@ int parse_opts(int argc, char** argv, sharp_conf_t* sharp_conf)
         option_index = 0;
         endptr = "\0";
 #ifdef MPI_SUPPORT
-        opt = getopt_long(argc, argv, "ht:d:m:c:i:j:H:x:B:N:M:p:s:r:D:T:C:f:", long_options, &option_index);
+        opt = getopt_long(argc, argv, "ht:d:m:c:R:i:j:H:x:B:N:M:p:s:r:D:T:C:f:", long_options, &option_index);
 #else
         opt = getopt_long(argc, argv, "hd:i:j:x:M:s:f:", long_options, &option_index);
 #endif
@@ -664,6 +668,8 @@ int parse_opts(int argc, char** argv, sharp_conf_t* sharp_conf)
                         sharp_conf->run_colls |= TEST_REDUCE_SCATTER;
                     } else if (strcmp(endptr, "allgather") == 0) {
                         sharp_conf->run_colls |= TEST_ALLGATHER;
+                    } else if (strcmp(endptr, "rs_ag") == 0) {
+                        sharp_conf->run_colls |= TEST_RS_AG;
                     } else if (strcmp(endptr, "none") == 0) {
                         sharp_conf->run_colls = 0;
                     } else if (strcmp(endptr, "all") == 0) {
@@ -675,6 +681,16 @@ int parse_opts(int argc, char** argv, sharp_conf_t* sharp_conf)
                     }
                 }
                 break;
+            case 'R':
+                if (strcmp(optarg, "seq") == 0) {
+                    sharp_conf->rs_ag_overlapped = 0;
+                } else if (strcmp(optarg, "ovl") == 0) {
+                    sharp_conf->rs_ag_overlapped = 1;
+                } else {
+                    fprintf(stderr, "Invalid --rs_ag_mode option. Use seq|ovl\n");
+                    res = INVALID_ARG;
+                }
+                break;
             case 'C':
                 tmp_id = strtol(optarg, &endptr, 10);
                 if (*endptr != '\0' || tmp_id < 0 || tmp_id > SHARP_COLL_DATA_MAX_IOV) {
