@@ -101,8 +101,8 @@ using std::vector;
 
 #define MAX_CONTROL_PATH_VERSION_SUPPORTED 2
 
-#define MIN_SHARP_MLID_START    0xC020
-#define MAX_SHARP_MLID_END      0xFFFF
+#define MIN_SHARP_MLID_START 0xC020
+#define MAX_SHARP_MLID_END   0xFFFF
 
 #define NON_PLANARIZED         ((uint8_t)0)
 #define PLANARIZED_FIRST_PLANE ((uint8_t)1)
@@ -208,6 +208,7 @@ using MulticastTableEntryVec = std::vector<MulticastTablePlftEntry>;
 // MulticastForwardingTable dimensions are: [mlid_block][port_group][plft]
 using MulticastForwardingTable = std::vector<MulticastTableEntryVec>;
 using MulticastTableUpdates = std::set<MulticastTableUpdate>;
+using Pkeys_t = std::set<uint16_t>;
 
 // messages of agg node states
 extern char const* const g_removed_from_network_msg;
@@ -339,7 +340,7 @@ struct HcaPortKey
     uint8_t m_plane;
 
     HcaPortKey() : m_key(0), m_plane(0) {}
-    HcaPortKey(uint64_t key, uint8_t plane) : m_key(key), m_plane(plane){};
+    HcaPortKey(uint64_t key, uint8_t plane) : m_key(key), m_plane(plane) {};
 
     uint64_t GetKey() const { return m_key; }
     std::string ToString() const;
@@ -654,6 +655,7 @@ struct PortInfo
     uint8_t m_last_plane;
     uint8_t m_max_plane;
     phys_port_t m_peer_port_num;
+    Pkeys_t m_pkeys;
 
     // PortInfo() : m_port_key(0), m_lid(0), m_peer_key(0) {}
 
@@ -669,6 +671,7 @@ struct PortInfo
              uint8_t last_plane,
              uint8_t max_plane,
              phys_port_t peer_port_num,
+             const Pkeys_t& pkeys,
              bool use_grh = false)
         : m_port_key(port_key),
           m_subnet_prefix(subnet_prefix),
@@ -682,7 +685,8 @@ struct PortInfo
           m_first_plane(first_plane),
           m_last_plane(last_plane),
           m_max_plane(max_plane),
-          m_peer_port_num(peer_port_num)
+          m_peer_port_num(peer_port_num),
+          m_pkeys(pkeys)
     {}
 
     string GetName() const
@@ -711,22 +715,7 @@ struct PortInfo
         return stream.str();
     }
 
-    string ToString() const
-    {
-        std::stringstream stream;
-        std::string lmc_str;
-        if (m_lmc) {
-            lmc_str = std::string(" lmc:") + std::to_string(m_lmc);
-        }
-        stream << m_node_desc;
-        if (IsPlanarized()) {
-            stream << " PLN:" << PlaneToString();
-        }
-        stream << " GUID:0x" << hex << m_port_key << " PEER GUID:0x" << m_peer_key << " lid:" << dec << m_lid << lmc_str << " "
-               << m_peer_topology_info.ToString();
-
-        return stream.str();
-    }
+    string ToString() const;
 
     port_key_t GetPortKey() const { return m_port_key; }
     port_key_t GetPeerPortKey() const { return m_peer_key; }
@@ -749,6 +738,8 @@ struct PortInfo
     };
     bool IsPlanarized() const { return m_first_plane && m_last_plane; };
     std::string PlaneToString() const;
+    const Pkeys_t& GetPkeys() const { return m_pkeys; }
+    void SetPkeys(const Pkeys_t& pkeys) { m_pkeys = pkeys; }
 };
 
 struct AggPortInfo
@@ -818,6 +809,7 @@ struct AggNodeInfo
     bool m_extended_data_types_supported;
     bool m_qp_to_port_select_supported;
     bool m_rmc_supported;
+    bool m_stochastic_rounding_supported;
     int m_clean_required;
     std::chrono::steady_clock::time_point m_clean_required_reset_timepoint;
 
@@ -863,6 +855,7 @@ struct AggNodeInfo
           m_extended_data_types_supported(false),
           m_qp_to_port_select_supported(false),
           m_rmc_supported(false),
+          m_stochastic_rounding_supported(false),
           m_clean_required(0),
           m_clean_required_reset_timepoint{},
           m_buffer_calculation_parameters()
@@ -936,7 +929,7 @@ class AnToAnInfo
     string m_prev_timestamp_;
 
    public:
-    AnToAnInfo() : m_key_(0, 0), m_port_num1_(0), m_port_num2_(0), m_prev_port_num1_(0), m_prev_port_num2_(0){};
+    AnToAnInfo() : m_key_(0, 0), m_port_num1_(0), m_port_num2_(0), m_prev_port_num1_(0), m_prev_port_num2_(0) {};
 
     AnToAnInfo(u_int64_t node_guid1, u_int64_t node_guid2, phys_port_t port_num1, phys_port_t port_num2, const string& timestamp)
         : m_key_(node_guid1, node_guid2), m_prev_port_num1_(0), m_prev_port_num2_(0), m_timestamp_(timestamp)
