@@ -120,17 +120,26 @@ class LogTimed
                         char const* const function_name,
                         CallbackType callback_to_invoke)
     {
-        ++m_number_of_requests_since_last_log_;
         if (!log_check_level(category, level)) {
             return;
         }
+
+        ++m_number_of_requests_since_last_log_;
         if ((0 != m_last_log_.time_since_epoch().count()) && (!ShouldLog())) {
             return;
         }
 
         m_last_log_ = std::chrono::steady_clock::now();
         const char* log_timed_format = "T[%zu]: %s";
+
+        // Invoke the callback, provide a string to it to fill
+        // Log only if there is text to write, the callback can return empty string, which means there is
+        // no content to log
         std::string str_to_log;
+        callback_to_invoke(str_to_log);
+        if (str_to_log.empty()) {
+            return;
+        }
 
         log_send(category,
                  level,
@@ -139,7 +148,7 @@ class LogTimed
                  function_name,
                  log_timed_format,
                  m_number_of_requests_since_last_log_,
-                 callback_to_invoke(str_to_log));
+                 str_to_log.c_str());
 
         m_number_of_requests_since_last_log_ = 0;
     }
