@@ -64,8 +64,8 @@ class ReservationJobInfo
     HashPortKeys m_guids;
 
    public:
-    ReservationJobInfo() : m_client_job_id_(0){};
-    ~ReservationJobInfo(){};
+    ReservationJobInfo() : m_client_job_id_(0) {};
+    ~ReservationJobInfo() {};
 
     void SetClientJobId(uint64_t client_job_id) { m_client_job_id_ = client_job_id; }
 
@@ -183,6 +183,7 @@ class ReservationManager
     ReservationInfo* GetReservationInfoByKey(const std::string& reservation_key);
 
     void ReadPersistentReservationInfoFiles();
+    void DeleteAllPersistentReservationFiles() const;
 
     sharp_am_status ValidateReservationResources(ReservationInfo* ri, JobResource& job_resource, const SharpJob* p_job);
 
@@ -192,6 +193,11 @@ class ReservationManager
 
     void RemoveJob(ReservationInfo* ri, JobInfos::iterator& it, const SharpJob* job);
 
+    void UpdateReservationWithGuidPkeys(hca_port_key_t port_key, const Pkeys_t& pkeys);
+
+    // Method to handle already running jobs for all reservations
+    void HandleAlreadyRunningJobsForAllReservations();
+
    private:
     enum sharp_reservation_status InternalUpdateReservation(ReservationInfo* ri, const sharp_create_reservation& create_reservation);
     void InternalCreateNewReservation(ReservationInfo* ri);
@@ -219,7 +225,7 @@ class ReservationManager
 
     void DeleteReservationInfoFile(const sharp_reservation_info& reservation_info, const string& persistent_path);
 
-    void DeleteFile(const string& file_path);
+    void DeleteFile(const string& file_path) const;
 
     void UpdateReservationInfoFile(const sharp_reservation_info& reservation_info, const string& persistent_path);
 
@@ -228,5 +234,15 @@ class ReservationManager
     int BuildDataFromReservationFile(const char* job_file, sharp_smx_msg*& smx_msg) const;
 
     void ApplyJobReservationResourcesChange(ReservationInfo* ri, bool is_added_job, const SharpJob* p_job);
+
+    // Internal function to update reservation without sending reply
+    enum sharp_reservation_status InternalUpdateReservationByDelta(const sharp_update_reservation_by_delta& update_reservation_by_delta);
+
+    // Helper methods for UpdateReservationWithGuidPkeys
+    void RemoveGuidFromReservation(ReservationInfo* ri, uint64_t guid);
+    void AddOrCreateReservationForGuid(uint64_t guid, uint16_t port_pkey);
+
+    // Helper method to handle already running jobs during reservation creation
+    void HandleAlreadyRunningJobs(ReservationInfo* ri);
 };
 #endif   // RESERVATION_MANAGER_H_
