@@ -153,8 +153,6 @@ class TreeEdge
 
     void DecExclusiveLockLoad();
 
-    uint8_t GetExclusiveLockLoad() { return m_exclusive_lock_load_; }
-
     void ClearInternalConfig()
     {
         m_qp_.Clear();
@@ -174,8 +172,6 @@ class TreeEdge
 
     void SetAggPath(AggPath* agg_path) { m_path_ = agg_path; }
 
-    bool IsPathValid() const { return (m_path_ && m_path_->IsAggPathValid()); }
-
     bool IsSatPathValid() const { return (m_path_ && m_path_->IsAggPathValid() && m_path_->GetSwHops() == 1); }
 
     bool IsAllocateQp() const { return (IsConfigOnFabricRequiredNoLogs(CHECK_PATH_IS_VALID) && m_qp_.GetState() == QP_STATE_UNALLOCATED); }
@@ -297,6 +293,7 @@ enum TreeNodeJobStateEnum
     TREE_NODE_JOB_STATE_CLEAN_QPS,                      // Disable QPs, remove from tree, unallocate QPs
     TREE_NODE_JOB_STATE_DELETE_TREE_NODE,               // TreeConfig TreeState 0, opcode 2
 };
+const char* ToString(TreeNodeJobStateEnum state);
 
 enum TreeNodeStateEnum
 {
@@ -305,6 +302,7 @@ enum TreeNodeStateEnum
     TREE_NODE_STATE_CONFIGURED,             // Configured successfully
     TREE_NODE_STATE_CONFIG_ERROR            // ERROR state. Error occurred after TreeConfig MAD with WRITE_TREE opcode sent
 };
+const char* ToString(TreeNodeStateEnum state);
 
 class AggNodeFabricInfo;
 
@@ -463,7 +461,6 @@ class TreeNode
     uint16_t GetJobsLoad() const { return m_jobs.size(); }
     void IncExclusiveLockLoad() { m_exclusive_lock_load_++; }
     void DecExclusiveLockLoad() { m_exclusive_lock_load_--; }
-    uint16_t GetExclusiveLockLoad() { return m_exclusive_lock_load_; }
 
     void SetJobState(TreeNodeJobStateEnum job_state) { m_job_state_ = job_state; }
 
@@ -473,8 +470,6 @@ class TreeNode
 
     int UnConfigureEdge(TreeEdge* p_tree_edge);
 
-    int ConfigureEdge(TreeEdge* p_tree_edge);
-
     void Recover();
 
     // return true if lock available, false otherwise.
@@ -503,8 +498,6 @@ class TreeNode
     */
     void TreeConfigDisable(int* p_operation_status);
 
-    void TryRecoverTree();
-
     int AssignRestoredQPsToTreeEdges();
 
     void RestoreAggPaths(int& operation_status);
